@@ -1,5 +1,5 @@
 package QuadPres;
-$QuadPres::VERSION = '0.28.4';
+$QuadPres::VERSION = '0.30.0';
 use 5.016;
 use strict;
 use warnings;
@@ -784,48 +784,54 @@ sub render
     return;
 }
 
-sub traverse_tree
+sub ref_traverse_tree
 {
     my $self     = shift;
     my $callback = shift;
 
     my $contents = $self->contents;
 
-    my $traverse_helper;
-    $traverse_helper = sub {
-        my $path_ref = shift;
-        my $coords   = shift;
-        my $branch   = shift;
+    (
+        sub {
+            my ( $path_ref, $coords, $branch ) = @_;
 
-        $callback->(
-            'path'   => $path_ref,
-            'branch' => $branch,
-            'coords' => $coords,
-        );
+            $callback->(
+                +{
+                    'path'   => $path_ref,
+                    'branch' => $branch,
+                    'coords' => $coords,
+                }
+            );
 
-        if ( exists( $branch->{'subs'} ) )
-        {
-            # Let's traverse all the directories
-            my $new_coord = 0;
-            foreach my $sub_branch ( @{ $branch->{'subs'} } )
+            if ( exists( $branch->{'subs'} ) )
             {
-                $traverse_helper->(
-                    [ @$path_ref, $sub_branch->{'url'} ],
-                    [ @$coords, $new_coord ], $sub_branch,
-                );
-            }
-            continue
-            {
-                $new_coord++;
+                # Let's traverse all the directories
+                my $new_coord = 0;
+                foreach my $sub_branch ( @{ $branch->{'subs'} } )
+                {
+                    __SUB__->(
+                        [ @$path_ref, $sub_branch->{'url'} ],
+                        [ @$coords, $new_coord ], $sub_branch,
+                    );
+                }
+                continue
+                {
+                    $new_coord++;
+                }
             }
         }
-    };
-
-    $traverse_helper->( [], [], $contents );
+    )->( [], [], $contents );
 
     return;
 }
 
+sub traverse_tree
+{
+    my ( $self, $cb ) = @_;
+
+    return $self->ref_traverse_tree( sub { return $cb->( %{ shift() } ); } );
+}
+
 sub get_breadcrumbs_trail
 {
     my $qp  = shift;
@@ -868,7 +874,7 @@ QuadPres - a presentation / slides manager.
 
 =head1 VERSION
 
-version 0.28.4
+version 0.30.0
 
 =head1 SYNOPSIS
 
@@ -1008,6 +1014,12 @@ TBD.
 
 TBD.
 
+=head2 ref_traverse_tree
+
+TBD.
+
+Added in version 0.30.0.
+
 =for :stopwords cpan testmatrix url bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 SUPPORT
